#
# Edit.py - DITrack module dealing with editions
#
# Copyright (c) 2006-2007 The DITrack Project, www.ditrack.org.
#
# $Id: Edit.py 1696 2007-07-10 22:45:31Z vss $
# $HeadURL: https://svn.xiolabs.com/ditrack/src/tags/0.8/DITrack/Edit.py $
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#  * Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

import os
import tempfile

import DITrack.Util.Misc

class EditorInvocationError(Exception):
    "Editor didn't finish successfully"
    pass

def edit_text(globals, text):
    """
    Create temporary file, dump 'text' into that and invoke user editor.
    Check if the editor is finished successfully, ask user if he is
    satisfied with the edition and return either new version of the text
    or the old one.
    """

    assert(len(globals.editor) > 0)

    (fd, name) = tempfile.mkstemp(prefix = "ditrack")
    os.write(fd, text)
    os.close(fd)

    # Invoke editor.
    args = [ globals.editor, name ]
    rc = DITrack.Util.Misc.spawnvp(os.P_WAIT, globals.editor, args)

    if rc:
        os.unlink(name)
        raise EditorInvocationError(rc)

    # Rewind and read.
    f = open(name)
    text = f.read()

    # Close and delete.
    f.close()
    os.unlink(name)

    return text
